import inspect
import os
import pdb
import sys

from django.conf import settings
from django.core.exceptions import MiddlewareNotUsed

from django_pdb.utils import get_ipdb, has_ipdb


class PdbMiddleware(object):
    """
    Middleware to break into pdb at the start of views.

    If `always_break` is set, due to `runserver --pdb` this will break
    into pdb at the start of every view.

    Otherwise it will break into pdb at the start of the view if
    a 'pdb' GET parameter is set on the request url.
    """

    always_break = False

    def __init__(self, debug_only=True):
        """
        If debug_only is True, this middleware removes itself
        unless settings.DEBUG is also True. Otherwise, this middleware
        is always active.
        """
        if debug_only and not settings.DEBUG:
            raise MiddlewareNotUsed()

    def get_type_pdb(self, request):
        type_pdb = None
        if self.always_break:
            type_pdb = self.always_break
        elif request.GET.get('pdb', None) is not None:
            type_pdb = 'pdb'
        elif request.GET.get('ipdb', None) is not None:
            type_pdb = 'ipdb'
        return type_pdb

    def process_view(self, request, view_func, view_args, view_kwargs):
        """
        If running with '--pdb', set a breakpoint at the start
        of each of each view before it gets called.
        """
        # Skip out unless using `runserver --pdb`,
        # or `pdb` is in the command line parameters
        type_pdb = self.get_type_pdb(request)
        if not type_pdb:
            return

        filename = inspect.getsourcefile(view_func)
        basename = os.path.basename(filename)
        dirname = os.path.basename(os.path.dirname(filename))
        lines, lineno = inspect.getsourcelines(view_func)
        temporary = True
        cond = None
        funcname = view_func.__name__

        print()
        print('{} {}'.format(request.method, request.get_full_path()))
        print('function "{}" in {}/{}:{}'.format(funcname,
            dirname, basename, lineno))
        print('args: {}'.format(view_args))
        print('kwargs: {}'.format(view_kwargs))
        print()

        if type_pdb == 'ipdb' and has_ipdb():
            p = get_ipdb()
        else:
            if not type_pdb == 'pdb':
                print('You do not install ipdb or ipython module')
            p = pdb.Pdb()
        p.reset()
        p.set_break(filename, lineno + 1, temporary, cond, funcname)
        sys.settrace(p.trace_dispatch)



# -*- coding: utf-8 -*-

import os

from django.core.management import find_commands

try:
    from django_pdb.apps import DjangoPdbConfig
    from django.apps import apps
except ImportError as e:
    # So django < 1.7
    from django.core.management import find_management_module
    import django_pdb.management

    def load_management_modules(commands):
        # Find the installed apps
        try:
            from django.conf import settings
            _apps = settings.INSTALLED_APPS
        except (AttributeError, EnvironmentError, ImportError):
            _apps = []

        # Find and load the management module for each installed app above
        # this one.
        for app_name in _apps:
            try:
                path = find_management_module(app_name)
                if path == django_pdb.management.__path__[0]:
                    # Found this app
                    break

                commands.update(
                    dict([(name, app_name) for name in find_commands(path)])
                )
            except ImportError:
                pass  # No management module - ignore this app
else:
    def load_management_modules(commands):
        app_configs = [app_config for app_config in apps.get_app_configs() if app_config.name != DjangoPdbConfig.name]
        for app_config in reversed(app_configs):
            path = os.path.join(app_config.path, 'management')
            commands.update({name: app_config.name for name in find_commands(path)})



import sys
import pdb

from django_pdb.management import load_parent_command
from django_pdb.middleware import PdbMiddleware

from optparse import make_option
from django_pdb.utils import has_ipdb
from django.views import debug


RunServerCommand = load_parent_command('runserver')


class Command(RunServerCommand):
    """
    Identical to Django's standard 'runserver' management command,
    except that it also adds support for a '--pdb' option.
    """
    option_list = RunServerCommand.option_list + (
        make_option('--pdb', action='store_true', dest='pdb', default=False,
            help='Drop into pdb shell on at the start of any view.'),
        make_option('--ipdb', action='store_true', dest='ipdb', default=False,
            help='Drop into ipdb shell on at the start of any view.'),
        make_option('--pm', action='store_true', dest='pm', default=False,
            help='Drop into ipdb shell if an exception is raised in a view.'),
    )

    def handle(self, *args, **options):
        # Add pdb middleware, if --pdb is specified, or if we're in DEBUG mode
        from django.conf import settings

        pdb_option = options.pop('pdb')
        ipdb_option = options.pop('ipdb')

        middleware = 'django_pdb.middleware.PdbMiddleware'
        if ((pdb_option or settings.DEBUG)
            and middleware not in settings.MIDDLEWARE_CLASSES):
            settings.MIDDLEWARE_CLASSES += (middleware,)

        self.pm = options.pop('pm')
        if self.pm:
            debug.technical_500_response = self.reraise

        # If --pdb is specified then always break at the start of views.
        # Otherwise break only if a 'pdb' query parameter is set in the url.
        if pdb_option:
            PdbMiddleware.always_break = 'pdb'
        elif ipdb_option:
            PdbMiddleware.always_break = 'ipdb'

        super(Command, self).handle(*args, **options)

    def reraise(self, request, exc_type, exc_value, tb):
        if has_ipdb():
            import ipdb
            p = ipdb
        else:
            p = pdb
        if self.pm:
            print >>sys.stderr, "Exception occured: %s, %s" % (exc_type, exc_value)
            p.post_mortem(tb)
        else:
            raise



from optparse import make_option
import sys

from django.core.management.commands import test

from django_pdb.management import load_parent_command
from django_pdb.testrunners import make_suite_runner


# Provide a Command class so that Django knows what will handle
# things. This module does not override it, so it just needs to find
# the parent Command.
Command = load_parent_command('test')


def patch_test_command(Command):
    """
    Monkeypatches Django's TestCommand so that it chooses to use
    ipdb or pdb, allowing subclasses to inherit from it and wrap its
    behaviour.
    """
    Command.option_list += type(Command.option_list)([
        make_option('--pdb', action='store_true', dest='pdb', default=False,
                    help='Drop into pdb shell on test errors or failures.'),
        make_option('--ipdb', action='store_true', dest='ipdb', default=False,
                    help='Drop into ipdb shell on test errors or failures.'),
    ])

    def handle(self, *test_labels, **options):
        """
        If --pdb is set on the command line ignore the default test runner
        use the pdb test runner instead.
        """
        pdb = options.pop('pdb')
        ipdb = options.pop('ipdb')

        if pdb or ipdb:
            options['verbosity'] = int(options.get('verbosity', 1))
            options['interactive'] = options.get('interactive', True)
            options['failfast'] = options.get('failfast', False)

            TestRunner = self.get_runner(use_ipdb=ipdb)
            test_runner = TestRunner(**options)
            failures = test_runner.run_tests(test_labels)

            if failures:
                sys.exit(bool(failures))

        else:
            self._handle(*test_labels, **options)

    Command._handle = Command.handle
    Command.handle = handle

    def get_runner(self, use_ipdb, suite_runner=None):
        return make_suite_runner(use_ipdb=use_ipdb, suite_runner=suite_runner)

    Command.get_runner = get_runner

patch_test_command(test.Command)



from django.core import management
from django.core.management import find_commands
from django.utils.importlib import import_module

from ..compat import load_management_modules

# A cache of loaded commands, so that call_command
# doesn't have to reload every time it's called.
_parent_commands = None


def get_parent_commands():
    """
    Returns a dictionary mapping command names to their callback applications.

    This function returns only callback applications above this
    application in the INSTALLED_APPS stack.
    """
    global _parent_commands
    if _parent_commands is None:
        django_path = management.__path__
        _parent_commands = dict([(name, 'django.core')
                                 for name in find_commands(django_path[0])])

        load_management_modules(_parent_commands)

        # Reset the Django management cache
        management._commands = None

    return _parent_commands


def load_parent_command(name):
    """
    Given a command name, returns the Command class instance that is
    the above the current application.

    (ImportError, AttributeError) are allowed to propagate.
    """
    app_name = get_parent_commands()[name]
    module = import_module('%s.management.commands.%s' % (app_name, name))
    return module.Command



from django import template
from django_pdb.utils import has_ipdb


register = template.Library()

@register.filter
def pdb(element):
    from django_pdb.utils import get_pdb_set_trace
    get_pdb_set_trace()()
    return element


@register.filter
def ipdb(element):
    if has_ipdb():
        from ipdb import set_trace
    else:
        from django_pdb.utils import get_pdb_set_trace
        get_pdb_set_trace()()
    set_trace()
    return element



from __future__ import print_function
from django.conf import settings
__version__ = '0.4.1'

POST_MORTEM = getattr(settings, 'POST_MORTEM', False)
DEBUG = getattr(settings, 'DEBUG', False)

if DEBUG and POST_MORTEM == True:
    from django.views import debug
    def runpdb(request, exc_type, exc_value, tb):
        import sys
        try:
            import ipdb
        except ImportError:
            import pdb as ipdb
        p = ipdb
        print('Exception occured: {}, {}'.format(exc_type, exc_value), file=sys.stderr)
        p.post_mortem(tb)
    debug.technical_500_response = runpdb

default_app_config = 'django_pdb.apps.DjangoPdbConfig'
